--[[
   KahLua KonferPUG - an open roll loot distribution helper for PUGs.
     WWW: http://kahluamod.com/kpug
     SVN: http://kahluamod.com/svn/konferpug
     IRC: #KahLua on irc.freenode.net
     E-mail: cruciformer@gmail.com
   Please refer to the file LICENSE.txt for the Apache License, Version 2.0.

   Copyright 2008-2009 James Kean Johnston. All rights reserved.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
]]

local K = LibStub:GetLibrary("KKore")

if (not K) then
  error ("KahLua KonferSK: could not find KahLua Kore.", 2)
end

local kpg = K:GetAddon ("KKonferPUG")
local L = kpg.L
local KUI = kpg.KUI
local MakeFrame = KUI.MakeFrame

-- Local aliases for global or Lua library functions
local _G = _G
local tinsert = table.insert
local tremove = table.remove
local setmetatable = setmetatable
local tconcat = table.concat
local tsort = table.sort
local tostring = tostring
local GetTime = GetTime
local min = math.min
local max = math.max
local strfmt = string.format
local strsub = string.sub
local strlen = string.len
local strfind = string.find
local xpcall, pcall = xpcall, pcall
local pairs, next, type = pairs, next, type
local select, assert, loadstring = select, assert, loadstring
local printf = K.printf
local ucolor = K.ucolor
local ecolor = K.ecolor
local icolor = K.icolor
local debug = kpg.debug
local info = kpg.info
local err = kpg.err

--
-- This file contains all of the UI initialisation code for KahLua KonferPUG.
--

local ignore
local sorted = {}

function kpg:InitialiseConfigGUI ()
  local arg
  local ypos = 0
  local cf = kpg.qf.lootopts

  arg = {
    x = 0, y = ypos,
    label = { text = L["Auto-open Loot Panel When Corpse Looted"] },
    tooltip = { title = "$$", text = L["TIP001"] },
  }
  cf.autoopen = KUI:CreateCheckBox (arg, cf)
  cf.autoopen:Catch ("OnValueChanged", function (this, evt, val)
    kpg.frdb.auto_open = val
  end)
  ypos = ypos - 24

  arg = {
    x = 0, y = ypos, label = { text = L["Display Tooltips in Loot List"] },
    tooltip = { title = "$$", text = L["TIP002"] },
  }
  cf.tooltips = KUI:CreateCheckBox (arg, cf)
  cf.tooltips:Catch ("OnValueChanged", function (this, evt, val)
    kpg.frdb.tooltips = val
  end)
  ypos = ypos - 24

  arg = {
    x = 0, y = ypos, label = { text = L["Announce Loot"] },
    tooltip = { title = "$$", text = L["TIP003"] },
  }
  cf.announce = KUI:CreateCheckBox (arg, cf)
  cf.announce:Catch ("OnValueChanged", function (this, evt, val)
    kpg.frdb.announce_loot = val
  end)

  arg = {
    x = 230, y = ypos, label = { text = L["Announce Winners in Raid"] },
    tooltip = { title = "$$", text = L["TIP007"] },
  }
  cf.winners = KUI:CreateCheckBox (arg, cf)
  cf.winners:Catch ("OnValueChanged", function (this, evt, val)
    kpg.frdb.announce_winners = val
  end)
  ypos = ypos - 24

  arg = {
    x = 0, y = ypos, label = { text = L["Auto-assign Loot When Roll Ends"] },
    tooltip = { title = "$$", text = L["TIP008"] },
  }
  cf.autoloot = KUI:CreateCheckBox (arg, cf)
  cf.autoloot:Catch ("OnValueChanged", function (this, evt, val)
    kpg.frdb.auto_loot = val
  end)
  ypos = ypos - 24

  arg = {
    x = 0, y = ypos,
    label = { text = L["Assign to Enchanter if no-one rolls"] },
    tooltip = { title = "$$", text = L["TIP009"] },
  }
  cf.disenchant = KUI:CreateCheckBox (arg, cf)
  cf.disenchant:Catch ("OnValueChanged", function (this, evt, val)
    kpg.frdb.disenchant = val
  end)
  ypos = ypos - 24

  arg = {
    x = 0, y = ypos, label = { text = L["Enable Chat Message Filter"] },
    tooltip = { title = "$$", text = L["TIP004"] },
  }
  cf.chatfilter = KUI:CreateCheckBox (arg, cf)
  cf.chatfilter:Catch ("OnValueChanged", function (this, evt, val)
    kpg.frdb.chat_filter = val
  end)
  ypos = ypos - 24

  arg = {
    x = 0, y = ypos, label = { text = L["Record Loot Assignment History"] },
  }
  cf.history = KUI:CreateCheckBox (arg, cf)
  cf.history:Catch ("OnValueChanged", function (this, evt, val, usr)
    if (usr and not val) then
      kpg.frdb.history = {}
      kpg:RefreshHistory ()
    end
    kpg.frdb.record_history = val
  end)
  ypos = ypos - 24

  arg = {
    x = 0, y = ypos, label = { text = L["Enable Off-spec Rolls"] },
    tooltip = { title = "$$", text = L["TIP024"] },
  }
  cf.offspec = KUI:CreateCheckBox (arg, cf)
  cf.offspec:Catch ("OnValueChanged", function (this, evt, val)
    kpg.frdb.offspec_rolls = val
    if (val) then
      kpg.qf.offspec:Show (val)
    else
      kpg.qf.offspec:Hide (val)
    end
    kpg.qf.switch:SetShown (val)
    kpg.qf.offtext:SetShown (val)
    kpg.qf.osmin:SetEnabled (val)
    kpg.qf.osmax:SetEnabled (val)
  end)
  ypos = ypos - 24

  arg = {
    label = { text = L["Roll Timeout"] },
    x = 0, y = ypos, minval = 5, maxval = 60,
    tooltip = { title = "$$", text = L["TIP005"] },
  }
  cf.rolltimeout = KUI:CreateSlider (arg, cf)
  cf.rolltimeout:Catch ("OnValueChanged", function (this, evt, newv)
    kpg.frdb.roll_timeout = newv
  end)

  arg = {
    x = 230, y = ypos, minval = 5, maxval = 30,
    label = { text = L["Roll Timeout Extension"] },
    tooltip = { title = "$$", text = L["TIP006"] },
  }
  cf.rollextend = KUI:CreateSlider (arg, cf)
  cf.rollextend:Catch ("OnValueChanged", function (this, evt, newv)
    kpg.frdb.roll_extend = newv
  end)
  ypos = ypos - 48

  arg = {
    x = 0, y = ypos, label = { text = L["Announce Countdown"] },
    tooltip = { title = "$$", text = L["TIP022"] },
  }
  cf.countdown = KUI:CreateCheckBox (arg, cf)
  cf.countdown:Catch ("OnValueChanged", function (this, evt, newv)
    kpg.qf.lootopts.only5:SetEnabled (newv)
    kpg.frdb.announce_countdown = newv
  end)

  arg = {
    x = 230, y = ypos, label = { text = L["Only at 5-second mark"] },
    tooltip = { title = "$$", text = L["TIP023"] },
  }
  cf.only5 = KUI:CreateCheckBox (arg, cf)
  cf.only5:Catch ("OnValueChanged", function (this, evt, newv)
    kpg.frdb.only_5secs = newv
  end)
  ypos = ypos - 24

  arg = {
    x = 0, y = ypos, label = { text = L["Main Spec Minimum"] },
    len = 4, numeric = true, initialvalue = tostring(kpg.frdb.main_spec_min),
    width = 32,
  }
  cf.msmin = KUI:CreateEditBox (arg, cf)
  cf.msmin:Catch ("OnValueChanged", function (this, evt, value)
    kpg.frdb.main_spec_min = tonumber (value)
  end)
  kpg.qf.msmin = cf.msmin

  arg = {
    x =  200, y = ypos, label = { text = L["Maximum"] },
    len = 4, numeric = true, initialvalue = tostring(kpg.frdb.main_spec_max),
    width = 32,
  }
  cf.msmax = KUI:CreateEditBox (arg, cf)
  cf.msmax:Catch ("OnValueChanged", function (this, evt, value)
    kpg.frdb.main_spec_max = tonumber (value)
  end)
  kpg.qf.msmax = cf.msmax
  ypos = ypos - 24

  arg = {
    x = 0, y = ypos, label = { text = L["Off-spec Minimum"] },
    len = 4, numeric = true, initialvalue = tostring(kpg.frdb.off_spec_min),
    width = 32,
  }
  cf.osmin = KUI:CreateEditBox (arg, cf)
  cf.osmin:Catch ("OnValueChanged", function (this, evt, value)
    kpg.frdb.off_spec_min = tonumber (value)
  end)
  kpg.qf.osmin = cf.osmin

  arg = {
    x =  200, y = ypos, label = { text = L["Maximum"] },
    len = 4, numeric = true, initialvalue = tostring(kpg.frdb.off_spec_max),
    width = 32,
  }
  cf.osmax = KUI:CreateEditBox (arg, cf)
  cf.osmax:Catch ("OnValueChanged", function (this, evt, value)
    kpg.frdb.off_spec_max = tonumber (value)
  end)
  kpg.qf.osmax = cf.osmax
  ypos = ypos - 24

  cf = kpg.qf.ignores
  cf.ilistframe = MakeFrame ("Frame", nil, cf)
  cf.ilistframe:SetWidth (370)
  cf.ilistframe:SetPoint ("TOPLEFT", cf, "TOPLEFT", 0, 0)
  cf.ilistframe:SetPoint ("BOTTOMLEFT", cf, "BOTTOMLEFT", 0, 0)

  arg = {
    name = "KPGIgnoreScrollIst",
    itemheight = 16,
    newitem = function (objp, num)
      return KUI.NewItemHelper (objp, num, "KPGIListButton", 350, 16,
        nil, nil, function (this, idx)
          kpg.qf.idelbtn:SetEnabled (true)
          ignore = sortedignores[idx]
        end)
      end,
    setitem = function (objp, idx, slot, btn)
      return KUI.SetItemHelper (objp, btn, idx,
        function (op, ix)
          return kpg.frdb.items[sortedignores[ix]]
        end)
      end,
    selectitem = function (objp, idx, slot, btn, onoff)
      return KUI.SelectItemHelper (objp, idx, slot, btn, onoff,
        function () return true end,
        function () if (kpg.qf.idelbtn) then
            kpg.qf.idelbtn:SetEnabled (true)
          end
        end,
        nil,
        function () if (kpg.qf.idelbtn) then
            kpg.qf.idelbtn:SetEnabled (false)
          end
        end)
      end,
    highlightitem = function (objp, idx, slot, btn, onoff)
      return KUI.HighlightItemHelper (objp, idx, slot, btn, onoff)
    end,
  }
  cf.ilist = KUI:CreateScrollList (arg, cf.ilistframe)
  kpg.qf.ignorelist = cf.ilist

  ypos = 0
  arg = {
    x = 375, y = ypos, text = L["Delete"], enabled = false,
    tooltip = { title = "$$", text = L["TIP021"] },
  }
  cf.idelbtn = KUI:CreateButton (arg, cf)
  kpg.qf.idelbtn = cf.idelbtn
  cf.idelbtn:Catch ("OnClick", function (this, evt)
    kpg.frdb.items[ignore] = nil
    ignore = nil
    kpg.qf.ignorelist:SetSelected (nil)
    kpg:RefreshIgnoredItems ()
  end)

  kpg:RefreshIgnoredItems ()
end

function kpg:RefreshIgnoredItems ()
  sortedignores = {}
  for k,v in pairs (kpg.frdb.items) do
    tinsert (sortedignores, k)
  end
  tsort (sortedignores, function (a,b)
    local an = GetItemInfo (kpg.frdb.items[a])
    local bn = GetItemInfo (kpg.frdb.items[b])
    return an < bn
  end)
  kpg.qf.ignorelist.itemcount = #sortedignores
  kpg.qf.ignorelist:UpdateList ()
end
